<?php
namespace NjFbCloud\Src\Classes;

use NjFbCloud\Src\Enum\CloudType;
class GoogleApiSyncFolders {

    private $root_folder_id = null;

    public function __construct( $root_folder_id )
    {
        $this->root_folder_id = $root_folder_id;
    }
    public function sync_folders_to_google_drive( $access_token ) {
        global $wpdb;
        $table = $wpdb->prefix . 'fbv';
        $root_folders = $wpdb->get_results("SELECT * FROM $table WHERE parent = 0 AND type = " . CloudType::GOOGLE_DRIVE);
        if(count($root_folders) > 0) {
            $child_folders = $wpdb->get_results("SELECT * FROM $table WHERE type = ".CloudType::GOOGLE_DRIVE." AND parent = {$root_folders[0]->id}");
            foreach($child_folders as $folder) {
                $this->process_folder($folder, $this->root_folder_id, $access_token);
            }
            // $this->delete_gd_folders( $root_folders[0]->id, $this->root_folder_id, $access_token );
        }
    }
    public function process_folder($folder, $parent_id, $access_token) {
        global $wpdb;
        $table = $wpdb->prefix . 'fbv';
    
        // Check if the folder already exists in Google Drive with the parent
        $existing_folder_id = $this->get_existing_folder_id($folder->name, $parent_id, $access_token);
    
        if (!$existing_folder_id) {
            // Folder doesn't exist, create it
            $post_fields = array(
                'name' => $folder->name,
                'mimeType' => 'application/vnd.google-apps.folder'
            );
    
            if ($parent_id) {
                $post_fields['parents'] = array($parent_id);
            }
    
            $post_fields_json = json_encode($post_fields);
    
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, "https://www.googleapis.com/drive/v3/files");
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $post_fields_json);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                "Authorization: Bearer $access_token",
                "Content-Type: application/json"
            ));
            $response = curl_exec($ch);
            curl_close($ch);
    
            $response_data = json_decode($response, true);
            if (isset($response_data['id'])) {
                $existing_folder_id = $response_data['id'];
            }
        }
        njfb_cloud_link_fb_to_cloud( 'google_drive', $folder->id, $existing_folder_id );
        
        // $this->delete_gd_folders( $folder->id, $existing_folder_id, $access_token );

        // Get the children of the current folder
        $child_folders = $wpdb->get_results("SELECT * FROM $table WHERE type = ".CloudType::GOOGLE_DRIVE." AND parent = {$folder->id}");
        foreach ($child_folders as $child_folder) {
            $this->process_folder($child_folder, $existing_folder_id, $access_token);
        }
    }
    
    public function delete_gd_folders($wp_folder_id, $gd_id, $access_token) {
        global $wpdb;
        $table = $wpdb->prefix . 'fbv';
        // Get the children of the current folder from WordPress
        $child_folders_wp = $wpdb->get_results("SELECT * FROM $table WHERE parent = {$wp_folder_id}");

        // Get the children of the current folder from Google Drive
        $child_folders_gd = $this->get_child_folders_from_drive($gd_id, $access_token);

        // Delete folders in Google Drive that are not in WordPress
        $wp_folder_names = array_map(function($child) {
            return $child->name;
        }, $child_folders_wp);

        foreach ($child_folders_gd as $child_folder_gd) {
            if (!in_array($child_folder_gd['name'], $wp_folder_names)) {
                $this->delete_folder_from_drive($child_folder_gd['id'], $access_token);
            }
        }
    }
    public function get_existing_folder_id($folder_name, $parent_id, $access_token) {
        $query = "mimeType='application/vnd.google-apps.folder' and name='$folder_name'";
    
        if ($parent_id) {
            $query .= " and '$parent_id' in parents";
        }
    
        $url = "https://www.googleapis.com/drive/v3/files?q=" . urlencode($query);
    
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Authorization: Bearer $access_token"
        ));
        $response = curl_exec($ch);
        curl_close($ch);
    
        $response_data = json_decode($response, true);
        if (isset($response_data['files']) && count($response_data['files']) > 0) {
            return $response_data['files'][0]['id'];
        } else {
            return null;
        }
    }
    public function get_child_folders_from_drive($parent_id, $access_token) {
        $child_folders = array();
        $query = "'$parent_id' in parents and mimeType='application/vnd.google-apps.folder'";
        $url = "https://www.googleapis.com/drive/v3/files?q=" . urlencode($query) . "&fields=files(id,name)";
    
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Authorization: Bearer $access_token"
        ));
        $response = curl_exec($ch);
        curl_close($ch);
    
        $response_data = json_decode($response, true);
        if (isset($response_data['files'])) {
            $child_folders = $response_data['files'];
        }
    
        return $child_folders;
    }
    public function delete_folder_from_drive($folder_id, $access_token) {
        $url = "https://www.googleapis.com/drive/v3/files/$folder_id";
    
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Authorization: Bearer $access_token"
        ));
        $response = curl_exec($ch);
        curl_close($ch);
    }
}