<?php
namespace NjFbCloud\Src\Cloud;

use NjFbCloud\Src\Classes\DropboxApi;
use NjFbCloud\Src\Enum\CloudType;

use NjFbCloud\Src\Classes\View;
class Dropbox extends Cloud {

    const ROOT_FOLDER_NAME = 'Dropbox';
    const ROOT_CLOUD_FOLDER_NAME = 'FileBird Cloud';


    public function __construct( $id )
    {
        parent::__construct( $id );
        add_action( 'wp_ajax_njfb_cloud_dropbox_disconnect', array( $this, 'ajaxDisconnect' ) );
        add_action( 'wp_ajax_njfb_cloud_dropbox_connected', array( $this, 'ajaxConnected' ) );

        add_action( 'wp_ajax_njfb_cloud_dropbox_resynchronize', array( $this, 'ajaxSyncFromCloudToFileBird' ) );
        add_action( 'wp_ajax_nopriv_njfb_cloud_dropbox_resynchronize', array( $this, 'ajaxSyncFromCloudToFileBird' ) );

        add_action( 'njfb_cloud_event', array( $this, 'syncFolderJob' ) );
    }

    public function syncFolderJob()
    {
        wp_remote_get( add_query_arg( array( 'action' => 'njfb_cloud_ggdrive_resynchronize' ), admin_url( 'admin-ajax.php' ) ) );
    }
    
    public function getLatestFiles( $folder_id = null )
    {
        
    }

    public function ajaxSyncFromCloudToFileBird()
    {
        set_time_limit(0);
        if( check_ajax_referer( 'njfb_cloud', 'nonce', false ) !== false || ( time() - (int)get_option( 'njfb_cloud_last_sync_' . $this->id, 0 ) ) > 1800 ) {
            try {
                $this->maybeRefreshToken();
            } catch ( \Exception $e ) {
                wp_send_json_error(['mess' => $e->getMessage()]);
            }
    
            $response_object = array(
                'success' => true
            );
            njfb_cloud_close_connection( $response_object );
            $this->doSync();
            update_option( 'njfb_cloud_last_sync_' . $this->id, time() );
        }
        exit();
    }

    public function doSync() {
        $ids = $this->maybeCreateRootFolder();
        $dropbox_folder_path = $ids['dropbox_folder_path'];
        $fb_folder_id = $ids['fb_folder_id'];
        $tree = $this->buildTreeStructure( $dropbox_folder_path );
        if( is_string( $tree ) ) {
            njfb_cloud_log('Cant sync DropBox: ' . $tree);
        } else {
            $this->syncToFileBird($tree, $fb_folder_id);
        }
    }
    private function syncToFileBird( $data, $parent_fb_id ) {
        if( is_array( $data) ) {
            foreach( $data as $file ) {
                $is_file = $file['.tag'] == 'file';
                if( $is_file ) {
                    $file_content = DropboxApi::getFileContent( $this->getBearerToken(), $file['path_lower'] );
                    //download image and assign to FileBird folder
                    $this->downloadAndAssignImage( $file['id'], $file['path_lower'], $file['name'], $file_content, $parent_fb_id );
                } else {
                    $fb_id = $this->insertOrGet( $file['name'], $parent_fb_id );
                    if( is_numeric( $fb_id ) && isset( $file['children'] ) && is_array( $file['children'] ) ) {
                        $this->syncToFileBird( $file['children'], $fb_id );
                    }
                }
            }
        }
    }
    private function downloadAndAssignImage( $file_id, $file_path, $file_name, $image_data, $filebird_folder ) {
        global $wpdb;

        //check if image exists
        $image = $wpdb->get_row( $wpdb->prepare( "SELECT `post_id` FROM " . $wpdb->postmeta . " WHERE `meta_key` = '_njfb_cloud_dropbox_id' AND `meta_value` = '%s'", $file_id ) );
        if( is_null( $image ) ) {
            $upload_dir = wp_upload_dir();
            if (wp_mkdir_p($upload_dir['path'])) {
                $file = wp_unique_filename( $upload_dir['path'], $file_name);
                $file = $upload_dir['path'] . '/' . $file;
            } else {
                $file = wp_unique_filename( $upload_dir['basedir'], $file_name);
                $file = $upload_dir['basedir'] . '/' . $file;
            }
            file_put_contents($file, $image_data);
            $wp_filetype = wp_check_filetype($file_name, null);
            $attachment = array(
                'post_mime_type' => $wp_filetype['type'],
                'post_title' => sanitize_file_name($file_name),
                'post_content' => '',
                'post_status' => 'inherit'
            );
            $attachment_id = wp_insert_attachment( $attachment, $file );
            if (!is_wp_error($attachment_id)) {
                require_once(ABSPATH . 'wp-admin/includes/image.php');
                $attachment_data = wp_generate_attachment_metadata($attachment_id, $file);
                wp_update_attachment_metadata($attachment_id, $attachment_data);
            }

            if( is_numeric( $attachment_id ) ) {
                update_post_meta( $attachment_id, '_njfb_cloud_dropbox_id', $file_id );
                update_post_meta( $attachment_id, '_njfb_cloud_dropbox_path', $file_path );
                $wpdb->insert( $wpdb->prefix . 'fbv_attachment_folder', array( 'folder_id' => $filebird_folder, 'attachment_id' => $attachment_id ), array( '%d', '%d' ) );
                clean_post_cache( $attachment_id );
            }
        } else {
            //assign to new folder
            $cleanQuery = $wpdb->prepare(
                "DELETE `fbva` FROM {$wpdb->prefix}fbv_attachment_folder AS `fbva`
                INNER JOIN {$wpdb->prefix}fbv AS `fbv`
                ON (fbva.folder_id = fbv.id AND fbv.created_by = %d)
                WHERE attachment_id = %d",
                apply_filters( 'fbv_folder_created_by', 0 ),
                $image->post_id
            );
            $wpdb->query( $cleanQuery );
            $wpdb->insert( $wpdb->prefix . 'fbv_attachment_folder', array( 'folder_id' => $filebird_folder, 'attachment_id' => $image->post_id ), array( '%d', '%d' ) );
        }
    }
    private function insertOrGet( $name, $parent_id ) {
        global $wpdb;
        $name  = sanitize_text_field( wp_unslash( wp_kses_post( $name ) ) );
        $check = $wpdb->get_row( $wpdb->prepare( "SELECT `id` FROM {$wpdb->prefix}fbv WHERE name = %s AND parent = %d AND created_by = %d", $name, $parent_id, apply_filters( 'fbv_folder_created_by', 0 ) ) );
        if( is_null( $check ) ) {
            $ord = $wpdb->get_var( $wpdb->prepare( "SELECT MAX(ord) FROM {$wpdb->prefix}fbv WHERE parent = %d AND created_by = %d", $parent_id, apply_filters( 'fbv_folder_created_by', 0 ) ) );
            $data = array(
                'name'       => $name,
                'parent'     => $parent_id,
                'type'       => 0,
                'created_by' => apply_filters( 'fbv_folder_created_by', 0 ),
                'ord'        => is_null( $ord ) ? 0 : ( intval( $ord ) + 1 ),
            );

            $inserted = $wpdb->insert( $wpdb->prefix . 'fbv', $data );
            return $inserted ? $wpdb->insert_id : false;
        } else {
            return $check->id;
        }
    }
    public function maybeCreateRootFolder()
    {
        global $wpdb;
        $fb_folder_id = (int)$wpdb->get_var( $wpdb->prepare( "SELECT `id` FROM {$wpdb->prefix}fbv WHERE `type` = %d AND `parent` = 0 AND `created_by` = %d", CloudType::DROPBOX, apply_filters( 'fbv_folder_created_by', 0 ) ) );
        if( $fb_folder_id === 0 ) {
            $wpdb->insert( $wpdb->prefix . 'fbv', array(
                'name' => self::ROOT_FOLDER_NAME,
                'parent' => '0',
                'type' => CloudType::DROPBOX,
                'ord' => '0',
                'created_by' => apply_filters( 'fbv_folder_created_by', 0 )
            ) );
            $fb_folder_id = $wpdb->insert_id;
        }
        $token = $this->getBearerToken();
        $dropbox_folder_path = njfb_cloud_get_main_folder( $this->id );
        if( empty( $dropbox_folder_path ) || !DropboxApi::isFolderAlive( $token, $dropbox_folder_path ) ) {
            $res = DropboxApi::createFolder( $token, self::ROOT_CLOUD_FOLDER_NAME . ' - ' . njfb_cloud_get_domain(), null );
            $dropbox_folder_path = $res['metadata']['path_lower'];
            njfb_cloud_set_main_folder( $this->id, $dropbox_folder_path );
        }
        return compact( "fb_folder_id", "dropbox_folder_path" );
    }

    public function ajaxDisconnect()
    {
        check_ajax_referer( 'njfb_cloud', 'nonce', true );
        DropboxApi::revoke( njfb_cloud_get_settings_by_key( $this->id, 'token', '' ) );
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', array() );
        if( ! is_array( $token ) ) {
            $token = array();
        }
        $refresh_token = isset( $token['refresh_token'] ) ? $token['refresh_token'] : '';
        njfb_cloud_set_settings_by_key( $this->id, 'token', array( 'refresh_token' => $refresh_token ) );
        wp_send_json_success();
    }

    public function ajaxConnected()
    {
        // https://www.dropbox.com/account/connected_apps
        $code = isset( $_GET['code'] ) ? $_GET['code'] : '';
        $token = DropboxApi::requestToken( $code );

        $error_mess = null;
        if( ! is_null( $token ) ) {
            if( isset( $token['error'] ) ) {
                $error_mess = $token['error'] . '. ' . $token['error_description'];
            } else {
                $token = njfb_cloud_append_refresh_token( $this->id, $token );
                njfb_cloud_set_settings_by_key( $this->id, 'token', $token );
                $this->maybeCreateRootFolder();
            }
        } else {
            $error_mess = 'Couldn\'t get token. Please try again.';
        }
        $arg = array('page' => NJFB_CLOUD_PAGE_SLUG, 'tab' => 'dropbox' );
        if( ! is_null( $error_mess ) ) {
            $arg['error_mess'] = $error_mess;
        }
        wp_safe_redirect( add_query_arg( $arg, admin_url( 'admin.php' ) ) );
        exit;
    }
    public function settingFields()
    {
        return array(
            array(
                'type' => 'custom_html',
                'html' => $this->connectButton()
            ),
            array(
                'type' => 'text',
                'name' => 'client_id',
                'label' => esc_html__( 'App Key', 'njfb_cloud' ),
                'description' => ''
            ),
            array(
                'type' => 'text',
                'name' => 'client_secret',
                'label' => esc_html__( 'App Secret', 'njfb_cloud' ),
                'description' => ''
            ),
            array(
                'type' => 'text',
                'name' => 'redirect_url',
                'label' => esc_html__( 'Redirect URIs', 'njfb_cloud' ),
                'description' => '',
                'value' => njfb_cloud_dropbox_redirecturis(),
                'attr' => array(
                    'readonly' => true
                )
            ),
        );
    }
    private function getAllFiles($path = '') {
        $token = $this->getBearerToken();

        $folder_content = DropboxApi::getFiles( $token, $path );
        if (is_string($folder_content)) {
            // It's an error message
            return $folder_content;
        }
    
        $all_entries = isset($folder_content['entries']) ? $folder_content['entries'] : array();
    
        // Continue fetching if there are more files
        while (isset($folder_content['has_more']) && $folder_content['has_more']) {
            if (!isset($folder_content['cursor'])) {
                break;
            }
    
            $folder_content = DropboxApi::getFilesContinue( $token, $folder_content['cursor'] );
    
            if (is_string($folder_content)) {
                // It's an error message
                return $folder_content;
            }
    
            if (isset($folder_content['entries'])) {
                $all_entries = array_merge($all_entries, $folder_content['entries']);
            }
        }
    
        return $all_entries;
    }
    
    private function buildTreeStructure($path = '') {
        $entries = $this->getAllFiles($path);
        if (is_string($entries)) {
            // It's an error message
            return $entries;
        }
    
        $tree = array();
        $folder_entries = array();
    
        // Separate folders and files
        foreach ($entries as $entry) {
            if ($entry['.tag'] === 'folder') {
                $folder_entries[$entry['path_lower']] = $entry;
            } else {
                $tree[] = $entry;
            }
        }
    
        // Recursively add children to folders
        foreach ($folder_entries as $folder_path => $folder_entry) {
            $folder_entry['children'] = $this->buildTreeStructure($folder_path);
            $tree[] = $folder_entry;
        }
    
        return $tree;
    }
    
    private function connectButton()
    {
        $client_id = njfb_cloud_get_settings_by_key( $this->id, 'client_id', '' );
        $client_secret = njfb_cloud_get_settings_by_key( $this->id, 'client_secret', '' );
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', '' );
        $auth_url = DropboxApi::authUrl();

        return View::load('admin/dropbox/connect', array(
            'client_id' => $client_id,
            'client_secret' => $client_secret,
            'token' => $token,
            'auth_url' => $auth_url
        ), false);
    }
    private function afterFbFolderCreated( $fb_folder_id, $fb_folder_name, $fb_folder_parent = null )
    {
        $dropbox_folder_id = njfb_cloud_get_cloud_id_from_fb( $this->id, $fb_folder_id );
        if( ! empty( $dropbox_folder_id ) ) {
            //check if folder is deleted, then recreate it
            if(false) {
                $dropbox_folder_id = '';
            }
        }
        if( empty( $dropbox_folder_id ) ) {
            $cloud_parent_id = null;
            if( ! is_null( $fb_folder_parent ) )  {
                $cloud_parent_id = njfb_cloud_get_cloud_id_from_fb( $this->id, $fb_folder_parent );
            }

            njfb_cloud_dispatch( 'DropboxJob', array(
                'job' => 'create',
                'args' => array(
                    'fb_id' => $fb_folder_id,
                    'name' => $fb_folder_name,
                    'parent' => $cloud_parent_id
                )
            ) );
        }
    }
    private function maybeRefreshToken()
    {
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', '' );
        if( ! is_array( $token ) || empty( $token) ) {
            throw new \Exception('Dropbox is not configured yet.');
        }
        if( DropboxApi::isExpired( $token ) && isset( $token['refresh_token'] ) ) {
            $new_token = DropboxApi::refreshToken( $token['refresh_token'] );
            if( ! is_null( $new_token ) ) {
                $new_token = njfb_cloud_append_refresh_token( $this->id, $new_token );
                njfb_cloud_set_settings_by_key( $this->id, 'token', $new_token );
            } else {
                throw new \Exception('Dropbox Token is invalid, please Disconnect then re-connect again.');
            }
        } else {
            if( ! isset( $token['refresh_token'] ) ) {
                throw new \Exception( 'Could not refresh Dropbox Token. Please Disconnect then re-connect again.' );
            }
        }
    }
    private function getBearerToken()
    {
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', '' );
        return (is_array( $token ) && isset( $token['access_token'] )) ? $token['access_token'] : '';
    }
}
