<?php
namespace NjFbCloud\Src\Cloud;

use NjFbCloud\Src\Classes\View;
use NjFbCloud\Src\Classes\GoogleDriveApi;
use NjFbCloud\Src\Classes\GoogleApiSyncFolders;
use NjFbCloud\Src\Enum\GoogleDriveMime;
use NjFbCloud\Src\Enum\CloudType;
class GoogleDrive extends Cloud {

    const ROOT_FOLDER_NAME = 'Google Drive';
    const ROOT_CLOUD_FOLDER_NAME = 'FileBird Cloud';

    public function __construct( $id )
    {
        parent::__construct( $id );
        add_action( 'wp_ajax_njfb_cloud_ggdrive_disconnect', array( $this, 'ajaxDisconnect' ) );
        add_action( 'wp_ajax_njfb_cloud_ggdrive_connected', array( $this, 'ajaxConnected' ) );

        add_action( 'wp_ajax_nopriv_njfb_cloud_ggdrive_webhook', array( $this, 'ajaxWebhook' ) );
        add_action( 'wp_ajax_nopriv_njfb_cloud_ggdrive_file_webhook', array( $this, 'ajaxFileWebhook' ) );

        add_action( 'wp_ajax_njfb_cloud_ggdrive_resynchronize', array( $this, 'ajaxSyncFromCloudToFileBird' ) );
        add_action( 'wp_ajax_nopriv_njfb_cloud_ggdrive_resynchronize', array( $this, 'ajaxSyncFromCloudToFileBird' ) );

        add_action( 'njfb_cloud_event', array( $this, 'syncFolderJob' ) );

        // add_action( 'njfb_cloud_event', array( $this, 'eventQueueWork' ) );
        // add_action( 'fbv_after_folder_renamed', array( $this, 'fbHookRenamed' ), 10, 2 );
        // add_action( 'fbv_after_folder_created', array( $this, 'fbHookCreated' ), 10, 2 );
        // add_action( 'fbv_after_folder_deleted', array( $this, 'fbHookDeleted' ) );
        // add_action( 'fbv_folder_parent_updated', array( $this, 'fbHookFolderParentUpdated' ), 10, 2 );
        // add_action( 'fbv_after_set_folder', array( $this, 'fbHookAfterSetFolder' ), 10, 2 );
        // add_action( 'fbv_after_assign_folder', array( $this, 'fbHookAfterAssignFolder' ), 10, 2 );
        // add_filter( 'fbv_can_delete_folder', array( $this, 'fbHookCanDeleteFolder' ), 10, 2 );
        // add_filter('wp_get_attachment_url', array( $this, 'externalMediaUrl' ), 10, 2 );
    }
    
    public function fbHookRenamed( $fb_folder_id, $new_name )
    {
        $cloud_id = njfb_cloud_get_cloud_id_from_fb( $this->id, $fb_folder_id );
        njfb_cloud_dispatch( 'GoogleDriveJob', array(
            'job' => 'rename',
            'args' => array(
                'id' => $cloud_id,
                'new_name' => $new_name
            )
        ) );
    }
    public function fbHookCreated( $fb_folder_id, $folder_data )
    {
        global $wpdb;
        if( (int)$folder_data['parent'] > 0 ) {
            $parent_folder = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}fbv WHERE `id` = %d", $folder_data['parent'] ) );
            if( $parent_folder && $parent_folder->type == CloudType::GOOGLE_DRIVE ) {
                $wpdb->update( $wpdb->prefix . 'fbv', array( 'type' => CloudType::GOOGLE_DRIVE ), array( 'id' => (int)$fb_folder_id ) );
                $this->afterFbFolderCreated( $fb_folder_id, $folder_data['title'], $folder_data['parent'] );
            }
        }
    }
    public function fbHookDeleted( $fb_folder_id )
    {
        $cloud_id = njfb_cloud_get_cloud_id_from_fb( $this->id, $fb_folder_id );
        njfb_cloud_dispatch( 'GoogleDriveJob', array(
            'job' => 'delete',
            'args' => array(
                'id' => $cloud_id
            )
        ) );
    }
    public function fbHookFolderParentUpdated( $fb_folder_id, $new_parent )
    {
        $cloud_id = njfb_cloud_get_cloud_id_from_fb( $this->id, $fb_folder_id );
        $new_parent_cloud = njfb_cloud_get_cloud_id_from_fb( $this->id, $new_parent );
        njfb_cloud_dispatch( 'GoogleDriveJob', array(
            'job' => 'update_parent',
            'args' => array(
                'id' => $cloud_id,
                'new_parent' => $new_parent_cloud
            )
        ) );
    }
    public function getLatestFiles( $folder_id = null, $page_token = null )
    {
        $res = GoogleDriveApi::getFiles( $this->getBearerToken(), $this->imageMimes(), array( 'mimeType' ), $folder_id, $page_token );
        return $res;
    }
    
    public function getAllFilesAndFolders($refresh_token = true, $parent = 'root', $page_token = null)
    {
        if( $refresh_token ) {
            $this->maybeRefreshToken();
        }
        $page_size = 100;

        $mimes = $this->imageMimes();
        $mimes[] = GoogleDriveMime::Folder;

        $res = GoogleDriveApi::getFiles( $this->getBearerToken(), $mimes, array( 'mimeType' ), $parent, $page_token, $page_size );
        foreach( $res['files'] as $k => $folder ) {
            $res['files'][$k]['children'] = $this->getAllFilesAndFolders( false, $folder['id'] );
        }
        if( ! is_null( $res['page_token'] ) ) {
            $res['files'] += $this->getAllFilesAndFolders( false, $parent, $res['page_token'] );
        }
        return $res;
    }
    
    public function settingFields()
    {
        // $html_folders = View::load( 'admin/ggdrive/folders', array( 'cloud' => $this->id, 'last_sync' => njfb_cloud_get_last_sync_folder( $this->id ) ), false );
        return array(
            array(
                'type' => 'custom_html',
                'html' => $this->connectButton()
            ),
            array(
                'type' => 'custom_html',
                'html' => $this->renderHiddenFields( 'token', njfb_cloud_get_settings_by_key( $this->id, 'token', '' ) )
            ),
            array(
                'type' => 'text',
                'name' => 'client_id',
                'label' => esc_html__( 'Google Client ID', 'njfb_cloud' ),
                'description' => ''
            ),
            array(
                'type' => 'text',
                'name' => 'client_secret',
                'label' => esc_html__( 'Google Client Secret', 'njfb_cloud' ),
                'description' => ''
            ),
            array(
                'type' => 'text',
                'name' => 'javascript_origins',
                'label' => esc_html__( 'JavaScript origins', 'njfb_cloud' ),
                'description' => '',
                'value' => njfb_cloud_ggdrive_jsorigins(),
                'attr' => array(
                    'readonly' => true
                )
            ),
            array(
                'type' => 'text',
                'name' => 'redirect_url',
                'label' => esc_html__( 'Redirect URIs', 'njfb_cloud' ),
                'description' => '',
                'value' => njfb_cloud_ggdrive_redirecturis(),
                'attr' => array(
                    'readonly' => true
                )
            ),
            // array(
            //     'type' => 'custom_html',
            //     'html' => '<div><textarea cols="100" rows="15">'.maybe_serialize( get_option('njfb_cloud_settings') ).'</textarea></div>'
            // ),
        );
    }
    public function ajaxDisconnect()
    {
        check_ajax_referer( 'njfb_cloud', 'nonce', true );
        // GoogleDriveApi::revokeAccess( njfb_cloud_get_settings_by_key( $this->id, 'token', '' ) );
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', array() );
        if( ! is_array( $token ) ) {
            $token = array();
        }
        $refresh_token = isset( $token['refresh_token'] ) ? $token['refresh_token'] : '';
        njfb_cloud_set_settings_by_key( $this->id, 'token', array( 'refresh_token' => $refresh_token ) );
        wp_send_json_success();
    }
    public function ajaxSyncFromCloudToFileBird() {
        set_time_limit(0);
        if( check_ajax_referer( 'njfb_cloud', 'nonce', false ) !== false || ( time() - (int)get_option( 'njfb_cloud_last_sync_' . $this->id, 0 ) ) > ( 12 * 1800) ) {
            try {
                $this->maybeRefreshToken();
            } catch ( \Exception $e ) {
                wp_send_json_error(['mess' => $e->getMessage()]);
            }

            $response_object = array(
                'success' => true
            );
            njfb_cloud_close_connection( $response_object );
            $this->doSync();
            update_option( 'njfb_cloud_last_sync_' . $this->id, time() );
        }
        exit();
    }
    public function doSync() {
        $ids = $this->maybeCreateRootFolder();
        $gg_folder_id = $ids['gg_folder_id'];
        $fb_folder_id = $ids['fb_folder_id'];

        $data = $this->getAllFilesAndFolders( true, $gg_folder_id );
        $this->syncToFileBird($data, $fb_folder_id);
    }
    private function syncToFileBird($data, $parent_fb_id) {
        $access_token = $this->getBearerToken();
        if( is_array( $data) && isset( $data['files'] ) && is_array( $data['files'] ) ) {
            foreach( $data['files'] as $file ) {
                $is_file = substr( $file['mimeType'], 0, 6 ) === 'image/';
                if( $is_file ) {
                    $image_data = GoogleDriveApi::getFileMediaContent( $access_token, $file['id']);
                    if( $image_data !== false ) {
                        //download image and assign to FileBird folder
                        $this->downloadAndAssignImage( $file['id'], $file['name'], $image_data, $parent_fb_id );
                    }
                } else {
                    $fb_id = $this->insertOrGet( $file['name'], $parent_fb_id );
                    if( is_numeric( $fb_id ) && isset( $file['children'] ) ) {
                        $this->syncToFileBird( $file['children'], $fb_id );
                    }
                }
            }
        }
    }
    private function downloadAndAssignImage( $file_id, $file_name, $image_data, $filebird_folder ) {
        global $wpdb;

        //check if image exists
        $image = $wpdb->get_row( $wpdb->prepare( "SELECT `post_id` FROM " . $wpdb->postmeta . " WHERE `meta_key` = '_njfb_cloud_gg_drive_id' AND `meta_value` = '%s'", $file_id ) );
        if( is_null( $image ) ) {
            $upload_dir = wp_upload_dir();
            if (wp_mkdir_p($upload_dir['path'])) {
                $file = wp_unique_filename( $upload_dir['path'], $file_name);
                $file = $upload_dir['path'] . '/' . $file;
            } else {
                $file = wp_unique_filename( $upload_dir['basedir'], $file_name);
                $file = $upload_dir['basedir'] . '/' . $file;
            }
            file_put_contents($file, $image_data);
            $wp_filetype = wp_check_filetype($file_name, null);
            $attachment = array(
                'post_mime_type' => $wp_filetype['type'],
                'post_title' => sanitize_file_name($file_name),
                'post_content' => '',
                'post_status' => 'inherit'
            );
            $attachment_id = wp_insert_attachment( $attachment, $file );
            if (!is_wp_error($attachment_id)) {
                require_once(ABSPATH . 'wp-admin/includes/image.php');
                $attachment_data = wp_generate_attachment_metadata($attachment_id, $file);
                wp_update_attachment_metadata($attachment_id, $attachment_data);
            }

            if( is_numeric( $attachment_id ) ) {
                update_post_meta( $attachment_id, '_njfb_cloud_gg_drive_id', $file_id );
                $wpdb->insert( $wpdb->prefix . 'fbv_attachment_folder', array( 'folder_id' => $filebird_folder, 'attachment_id' => $attachment_id ), array( '%d', '%d' ) );
                clean_post_cache( $attachment_id );
            }
        } else {
            //assign to new folder
            $cleanQuery = $wpdb->prepare(
                "DELETE `fbva` FROM {$wpdb->prefix}fbv_attachment_folder AS `fbva`
                INNER JOIN {$wpdb->prefix}fbv AS `fbv`
                ON (fbva.folder_id = fbv.id AND fbv.created_by = %d)
                WHERE attachment_id = %d",
                apply_filters( 'fbv_folder_created_by', 0 ),
                $image->post_id
            );
            $wpdb->query( $cleanQuery );
            $wpdb->insert( $wpdb->prefix . 'fbv_attachment_folder', array( 'folder_id' => $filebird_folder, 'attachment_id' => $image->post_id ), array( '%d', '%d' ) );
        }
    }
    private function downloadAndAssignImage_old( $file_id, $file_url, $file_name, $filebird_folder ) {
        global $wpdb;

        //check if image exists
        $image = $wpdb->get_row( $wpdb->prepare( "SELECT `post_id` FROM " . $wpdb->postmeta . " WHERE `meta_key` = '_njfb_cloud_gg_drive_id' AND `meta_value` = '%s'", $file_id ) );
        if( is_null( $image ) ) {
            $file_array         = array();
            $file_array['name'] = $file_name;

            // Download file to temp location.
            $file_array['tmp_name'] = download_url( $file_url );

            if ( is_wp_error( $file_array['tmp_name'] ) ) {
                return $file_array['tmp_name'];
            }

            // Do the validation and storage stuff.
            $image_id = media_handle_sideload( $file_array );

            // $image_id = media_sideload_image( $file_url, 0, null, 'id' );
            if( is_numeric( $image_id ) ) {
                update_post_meta( $image_id, '_njfb_cloud_gg_drive_id', $file_id );
                $wpdb->insert( $wpdb->prefix . 'fbv_attachment_folder', array( 'folder_id' => $filebird_folder, 'attachment_id' => $image_id ), array( '%d', '%d' ) );
                clean_post_cache( $image_id );
            }
        } else {
            //assign to new folder
            $cleanQuery = $wpdb->prepare(
                "DELETE `fbva` FROM {$wpdb->prefix}fbv_attachment_folder AS `fbva`
                INNER JOIN {$wpdb->prefix}fbv AS `fbv`
                ON (fbva.folder_id = fbv.id AND fbv.created_by = %d)
                WHERE attachment_id = %d",
                apply_filters( 'fbv_folder_created_by', 0 ),
                $image->post_id
            );
            $wpdb->query( $cleanQuery );
            $wpdb->insert( $wpdb->prefix . 'fbv_attachment_folder', array( 'folder_id' => $filebird_folder, 'attachment_id' => $image->post_id ), array( '%d', '%d' ) );
        }
    }
    private function insertOrGet( $name, $parent_id ) {
        global $wpdb;
        $name  = sanitize_text_field( wp_unslash( wp_kses_post( $name ) ) );
        $check = $wpdb->get_row( $wpdb->prepare( "SELECT `id` FROM {$wpdb->prefix}fbv WHERE name = %s AND parent = %d AND created_by = %d", $name, $parent_id, apply_filters( 'fbv_folder_created_by', 0 ) ) );
        if( is_null( $check ) ) {
            $ord = $wpdb->get_var( $wpdb->prepare( "SELECT MAX(ord) FROM {$wpdb->prefix}fbv WHERE parent = %d AND created_by = %d", $parent_id, apply_filters( 'fbv_folder_created_by', 0 ) ) );
            $data = array(
                'name'       => $name,
                'parent'     => $parent_id,
                'type'       => 0,
                'created_by' => apply_filters( 'fbv_folder_created_by', 0 ),
                'ord'        => is_null( $ord ) ? 0 : ( intval( $ord ) + 1 ),
            );

            $inserted = $wpdb->insert( $wpdb->prefix . 'fbv', $data );
            return $inserted ? $wpdb->insert_id : false;
        } else {
            return $check->id;
        }
    }
    // public function ajaxSyncFromFileBirdToCloud()
    // {
    //     set_time_limit(0);
    //     check_ajax_referer( 'njfb_cloud', 'nonce', true );
        
    //     njfb_cloud_clear_jobs( 'GoogleDriveJob' );
    //     njfb_cloud_clear_options( $this->id );

    //     $this->maybeRefreshToken();
    //     $access_token = $this->getBearerToken();
        
    //     $fb_folder_id = $this->createRootFBFolder();
    //     $res = GoogleDriveApi::createFolder( $access_token, $this->rootGDFolderName() );
    //     njfb_cloud_link_fb_to_cloud( $this->id, $fb_folder_id, $res['id'] );
    //     $root_gd_folder_id = $res['id'];
        
    //     $sync_instance = new GoogleApiSyncFolders( $root_gd_folder_id );
    //     $sync_instance->sync_folders_to_google_drive( $access_token );

    //     wp_send_json_success();
    // }
    public function syncFolderJob()
    {
        wp_remote_get( add_query_arg( array( 'action' => 'njfb_cloud_ggdrive_resynchronize' ), admin_url( 'admin-ajax.php' ) ) );
    }
    public function ajaxConnected()
    {
        // manage connected apps here https://myaccount.google.com/connections
        $code = isset( $_GET['code'] ) ? $_GET['code'] : '';
        $token = GoogleDriveApi::requestToken( $code );

        $error_mess = null;
        if( ! is_null( $token ) ) {
            if( isset( $token['error'] ) ) {
                $error_mess = $token['error'] . '. ' . $token['error_description'];
            } else {
                $token = njfb_cloud_append_refresh_token( $this->id, $token );
                njfb_cloud_set_settings_by_key( $this->id, 'token', $token );
                // $this->subscribeDrive();
                $this->maybeCreateRootFolder();
            }
        } else {
            $error_mess = 'Couldn\'t get token. Please try again.';
        }
        $arg = array('page' => NJFB_CLOUD_PAGE_SLUG, 'tab' => 'google_drive' );
        if( ! is_null( $error_mess ) ) {
            $arg['error_mess'] = $error_mess;
        }
        wp_safe_redirect( add_query_arg( $arg, admin_url( 'admin.php' ) ) );
        exit;
    }
    public function ajaxWebhook()
    {
        if (isset($_SERVER['HTTP_X_GOOG_CHANNEL_ID']) && isset($_SERVER['HTTP_X_GOOG_RESOURCE_ID'])) {
            $channelId = $_SERVER['HTTP_X_GOOG_CHANNEL_ID'];
            $resourceId = $_SERVER['HTTP_X_GOOG_RESOURCE_ID'];
        
            // Your logic to handle the incoming notification
            error_log('webhook: ' . print_r(['channelId' => $channelId, 'resourceId' => $resourceId], true));
            error_log( 'webhook POST: ' . file_get_contents("php://input"));
            error_log( 'webhook SERVER: ' . print_r($_SERVER, true));
            wp_send_json_success();
        } else {
            wp_send_json_error( null, 400 ); // Bad Request
        }
    }
    public function ajaxFileWebhook()
    {
        error_log( 'File Webhook POST 1: ' . print_r($_POST, true));
        error_log( 'File Webhook POST 2: ' . file_get_contents("php://input"));
        error_log( 'File Webhook SERVER: ' . print_r($_SERVER, true));
    }
    public function maybeCreateRootFolder()
    {
        $fb_folder_id = $this->createRootFBFolder();

        $gg_folder_id = njfb_cloud_get_main_folder( $this->id );
        $token = $this->getBearerToken();
        if( empty( $gg_folder_id ) || !GoogleDriveApi::isFolderAlive( $token, $gg_folder_id) ) {
            $res = GoogleDriveApi::createFolder( $token, $this->rootGDFolderName(), null );
            $gg_folder_id = $res['id'];
            njfb_cloud_set_main_folder( $this->id, $gg_folder_id );
        }
        return compact( "fb_folder_id", "gg_folder_id" );
    }
    public function subscribeDrive()
    {
        $this->maybeRefreshToken();
        $address = add_query_arg( array( 'action' => 'njfb_cloud_ggdrive_webhook' ), admin_url( 'admin-ajax.php' ) );
        $sub = GoogleDriveApi::subscribeToDriveChanges( $this->getBearerToken(), $address );
        error_log('subscribed: ' . print_r($sub, true));
        njfb_cloud_save_ggdrive_sub( $sub );
    }
    public function listChanges()
    {
        $this->maybeRefreshToken();
        $token = $this->getBearerToken();
        $pageToken = GoogleDriveApi::getStartPageToken( $token );

        $changes = array();

        $res = GoogleDriveApi::getChanges( $token, $pageToken );
        while( count( $res['changes'] ) > 0 ) {
            $changes = array_merge( $changes, $res['changes'] );
            $res = GoogleDriveApi::getChanges( $token, $res['newStartPageToken'] );
        }
        return $changes;
    }
    private function rootGDFolderName()
    {
        return self::ROOT_CLOUD_FOLDER_NAME . ' - ' . njfb_cloud_get_domain();
    }
    private function createRootFBFolder()
    {
        global $wpdb;
        $fb_folder_id = (int)$wpdb->get_var( $wpdb->prepare( "SELECT `id` FROM {$wpdb->prefix}fbv WHERE `type` = %d AND `parent` = 0 AND `created_by` = %d", CloudType::GOOGLE_DRIVE, apply_filters( 'fbv_folder_created_by', 0 ) ) );
        if( $fb_folder_id === 0 ) {
            $wpdb->insert( $wpdb->prefix . 'fbv', array(
                'name' => self::ROOT_FOLDER_NAME,
                'parent' => '0',
                'type' => CloudType::GOOGLE_DRIVE,
                'ord' => '0',
                'created_by' => apply_filters( 'fbv_folder_created_by', 0 )
            ) );
            $fb_folder_id = $wpdb->insert_id;
        }
        return $fb_folder_id;
    }
    public function fbHookAfterSetFolder( $post_id, $folder_id )
    {
        $this->afterAssignMediaToFolder( $post_id, $folder_id );
    }
    public function fbHookAfterAssignFolder( $folder_id, $post_ids )
    {
        foreach( $post_ids as $post_id ) {
            $this->afterAssignMediaToFolder( $post_id, $folder_id );
        }
    }
    
    public function externalMediaUrl( $wp_get_attachment_url, $attachment_id )
    {
        $gg_drive_url = get_post_meta( $attachment_id, '_njfb_cloud_gg_drive_url', true );
        if( filter_var( $gg_drive_url, FILTER_VALIDATE_URL ) ) {
            // $wp_get_attachment_url = sprintf( 'https://drive.usercontent.google.com/download?id=%s', njfb_cloud_get_ggdrive_id( $gg_drive_url ) );
            $wp_get_attachment_url = sprintf( 'https://lh3.googleusercontent.com/d/%s', njfb_cloud_get_ggdrive_id( $gg_drive_url ) );
        }
        return $wp_get_attachment_url;
    }
    public function fbHookCanDeleteFolder( $can, $folder_id )
    {
        $detail = \FileBird\Model\Folder::findById( $folder_id, 'type, parent' );
        if( ! is_null( $detail ) && $detail->type == CloudType::GOOGLE_DRIVE && $detail->parent == 0 ) {
            $can = false;
        }
        return $can;
    }
    public function eventQueueWork()
    {
        njfb_cloud_queue_work();
    }
    private function afterAssignMediaToFolder( $post_id, $folder_id )
    {
        $detail = \FileBird\Model\Folder::findById( $folder_id, 'type' );
        if( ! is_null( $detail ) && $detail->type == CloudType::GOOGLE_DRIVE ) {
            $gg_drive_url = get_post_meta( $post_id, '_njfb_cloud_gg_drive_url', true);
            if( ! filter_var( $gg_drive_url, FILTER_VALIDATE_URL ) ) {
                $image_path = get_attached_file( $post_id );
                if( $image_path && file_exists( $image_path ) ) {
                    $filesize = filesize( $image_path );
                    if( $filesize <= 5000000 ) {
                        $gg_folder_id = njfb_cloud_get_cloud_id_from_fb( $this->id, $folder_id );
                        if( $gg_folder_id ) {
                            njfb_cloud_dispatch( 'GoogleDriveJob', array(
                                'job' => 'upload_multipart',
                                'args' => array(
                                    'path' => $image_path,
                                    'parent' => $gg_folder_id,
                                    'fb_id' => $post_id
                                )
                            ) );
                        }
                    } else {
                        //TODO upload large file by queue
                    }
                }
            } else {
                $drive_id = njfb_cloud_get_ggdrive_id( $gg_drive_url );
                if( ! is_null( $drive_id ) ) {
                    $gg_folder_id = njfb_cloud_get_cloud_id_from_fb( $this->id, $folder_id );
                    //TODO check if file exists then update parent, if not, upload new file
                    njfb_cloud_dispatch( 'GoogleDriveJob', array(
                        'job' => 'update_parent',
                        'args' => array(
                            'id' => $drive_id,
                            'new_parent' => $gg_folder_id
                        )
                    ) );
                } else {
                    njfb_cloud_log( "Could not move this file to new parent: " . $gg_drive_url );
                }
            }
        } else {
            //remove from Google Drive
            $gg_drive_url = get_post_meta( $post_id, '_njfb_cloud_gg_drive_url', true);
            if( filter_var( $gg_drive_url, FILTER_VALIDATE_URL ) ) {
                $drive_id = njfb_cloud_get_ggdrive_id( $gg_drive_url );
                if( ! is_null( $drive_id ) ) {
                    njfb_cloud_dispatch( 'GoogleDriveJob', array(
                        'job' => 'delete',
                        'args' => array(
                            'id' => $drive_id,
                        )
                    ) );
                }
            }
        }
    }
    private function afterFbFolderCreated( $fb_folder_id, $fb_folder_name, $fb_folder_parent = null )
    {
        $gd_folder_id = njfb_cloud_get_cloud_id_from_fb( $this->id, $fb_folder_id );
        if( empty( $gd_folder_id ) ) {
            $cloud_parent_id = null;
            if( ! is_null( $fb_folder_parent ) )  {
                $cloud_parent_id = njfb_cloud_get_cloud_id_from_fb( $this->id, $fb_folder_parent );
            }
            $res = GoogleDriveApi::createFolder( $this->getBearerToken(), $fb_folder_name, $cloud_parent_id );
            $gd_folder_id = $res['id'];
            njfb_cloud_link_fb_to_cloud( 'google_drive', $fb_folder_id, $gd_folder_id );

            // njfb_cloud_dispatch( 'GoogleDriveJob', array(
            //     'job' => 'create',
            //     'args' => array(
            //         'fb_id' => $fb_folder_id,
            //         'name' => $fb_folder_name,
            //         'parent' => $cloud_parent_id
            //     )
            // ) );
        }
        return $gd_folder_id;
    }
    private function connectButton()
    {
        $client_id = njfb_cloud_get_settings_by_key( $this->id, 'client_id', '' );
        $client_secret = njfb_cloud_get_settings_by_key( $this->id, 'client_secret', '' );
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', '' );
        $auth_url = GoogleDriveApi::authUrl();

        return View::load('admin/ggdrive/connect', array('client_id' => $client_id, 'client_secret' => $client_secret, 'token' => $token, 'auth_url' => $auth_url), false);
    }
    private function isConnected()
    {
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', '' );
        return is_array( $token ) && ! empty( $token );
    }
    private function isDisconnected()
    {
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', '' );
        return ! is_array( $token ) || empty( $token );
    }
    
    private function getBearerToken()
    {
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', '' );
        return (is_array( $token ) && isset( $token['access_token'] )) ? $token['access_token'] : '';
    }
    private function maybeRefreshToken()
    {
        $token = njfb_cloud_get_settings_by_key( $this->id, 'token', '' );
        if( ! is_array( $token ) || empty( $token) ) {
            throw new \Exception('Google Drive is not configured yet.');
        }
        if( GoogleDriveApi::isExpired( $token ) && isset( $token['refresh_token'] ) ) {
            $new_token = GoogleDriveApi::refreshToken( $token['refresh_token'] );
            if( ! is_null( $new_token ) ) {
                $new_token = njfb_cloud_append_refresh_token( $this->id, $new_token );
                njfb_cloud_set_settings_by_key( $this->id, 'token', $new_token );
            } else {
                throw new \Exception('Google Drive Token is invalid, please Disconnect then re-connect again.');
            }
        } else {
            if( ! isset( $token['refresh_token'] ) ) {
                throw new \Exception( 'Could not refresh Google Drive Token. Please Disconnect then re-connect again.' );
            }
        }
    }
    private function toFbFormat( $from_folders )
    {
        $to_folders = array();
        foreach( $from_folders as $folder ) {
            $to_folders[] = array(
                'cloud_id' => $folder['id'],
                'name' => $folder['name'],
                'createdTime' => $folder['createdTime'],
                'modifiedTime' => $folder['modifiedTime'],
            );
        }
        return $to_folders;
    }
    private function imageMimes()
    {
        return array(
            GoogleDriveMime::Png,
            GoogleDriveMime::Jpeg,
            GoogleDriveMime::Jpg,
            GoogleDriveMime::Gif,
            GoogleDriveMime::Webp,
            GoogleDriveMime::Svg,
        );
    }
}
