<?php
namespace NjFbCloud\Src\Jobs;

use NjFbCloud\Src\Classes\DropboxApi;

class DropboxJob extends Job {

    private $args;

    public function __construct( $args )
    {
        $this->args = wp_parse_args( $args, array(
            'job' => '',
            'args' => array()
        ) );
        
        if( $this->args['job'] == 'upload' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'path' => '',
                'parent' => null,
                'position' => 0,
                'length' => null
            ) );
        } elseif( $this->args['job'] == 'rename' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'id' => '',
                'fb_id' => '',
                'new_name' => '',
            ) );
        } elseif( $this->args['job'] == 'create' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'name' => '',
                'parent' => '',
                'fb_id' => ''
            ) );
        } elseif( $this->args['job'] == 'delete' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'id' => ''
            ) );
        } elseif( $this->args['job'] == 'update_parent' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'id' => '',
                'new_parent' => ''
            ) );
        } elseif( $this->args['job'] == 'upload_multipart' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'path' => '',
                'parent' => null,
                'fb_id' => ''
            ) );
        }
    }
    public function handle() {
        $job = $this->args['job'];
        $args = $this->args['args'];
        
        if( $job == 'create' ) {
            if( ! empty( $args['id'] ) ) {
                if( $this->maybeRefreshToken() === false ) {
                    return false;
                }
                $created = DropboxApi::createFolder( $this->getBearerToken(), $args['name'], null );
            }
        }
        return true;
    }
    private function getBearerToken()
    {
        $token = njfb_cloud_get_settings_by_key( 'dropbox', 'token', '' );
        return (is_array( $token ) && isset( $token['access_token'] )) ? $token['access_token'] : '';
    }
    private function maybeRefreshToken()
    {
        $token = njfb_cloud_get_settings_by_key( 'dropbox', 'token', '' );
        if( DropboxApi::isExpired( $token ) && isset( $token['refresh_token'] ) ) {
            $new_token = DropboxApi::refreshToken( $token['refresh_token'] );
            if( ! is_null( $new_token ) ) {
                $new_token = njfb_cloud_append_refresh_token( 'dropbox', $new_token );
                njfb_cloud_set_settings_by_key( 'dropbox', 'token', $new_token );
                return true;
            } else {
                return false;
            }
        }
        return true;
    }
}
