<?php
namespace NjFbCloud\Src\Jobs;

use NjFbCloud\Src\Classes\GoogleApiUploadResumable;
use NjFbCloud\Src\Classes\GoogleDriveApi;

class GoogleDriveJob extends Job {

    private $args;

    public function __construct( $args )
    {
        $this->args = wp_parse_args( $args, array(
            'job' => '',
            'args' => array()
        ) );
        
        if( $this->args['job'] == 'upload' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'path' => '',
                'parent' => null,
                'position' => 0,
                'length' => null
            ) );
        } elseif( $this->args['job'] == 'rename' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'id' => '',
                'fb_id' => '',
                'new_name' => '',
            ) );
        } elseif( $this->args['job'] == 'create' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'name' => '',
                'parent' => '',
                'fb_id' => ''
            ) );
        } elseif( $this->args['job'] == 'delete' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'id' => ''
            ) );
        } elseif( $this->args['job'] == 'update_parent' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'id' => '',
                'new_parent' => ''
            ) );
        } elseif( $this->args['job'] == 'upload_multipart' ) {
            $this->args['args'] = wp_parse_args( $this->args['args'], array(
                'path' => '',
                'parent' => null,
                'fb_id' => ''
            ) );
        }
    }
    public function handle() {
        $job = $this->args['job'];
        $args = $this->args['args'];
        
        if( $job == 'upload' ) {
            if( $this->maybeRefreshToken() === false ) {
                return false;
            }
            $args['token'] = $this->getBearerToken();

            $uploader = new GoogleApiUploadResumable( $args );
            $uploader->setPosition( $args['position'] );
            if( ! is_null( $args['length'] ) ) {
                $uploader->setLength( $args['length'] );
            }
            $res = $uploader->uploadChunk();
            if( $res['finish'] !== true ) {
                if( isset( $res['position'] ) && isset( $res['length'] ) ) {
                    njfb_cloud_dispatch( 'GoogleDriveJob', array(
                        'job' => 'upload',
                        'args' => array(
                            'path' => $args['path'],
                            'parent' => $args['parent'],
                            'position' => $res['position'],
                            'length' => $res['length'],
                        )
                    ) );
                }
            } else {
                // finish
            }
            return true;
        } elseif( $job == 'rename' ) {
            if( ! empty( $args['id'] ) && ! empty( $args['new_name'] ) ) {
                if( $this->maybeRefreshToken() === false ) {
                    return false;
                }
                GoogleDriveApi::renameFolder( $this->getBearerToken(), $args['id'], $args['new_name'] );
            }
        }  elseif( $job == 'delete' ) {
            if( ! empty( $args['id'] ) ) {
                if( $this->maybeRefreshToken() === false ) {
                    return false;
                }
                GoogleDriveApi::deleteFolder( $this->getBearerToken(), $args['id'] );
                njfb_cloud_remove_link_fb( 'google_drive', $args['id'] );
            }
        } elseif( $job == 'update_parent' ) {
            if( ! empty( $args['id'] ) && ! empty( $args['new_parent'] ) ) {
                if( $this->maybeRefreshToken() === false ) {
                    return false;
                }
                GoogleDriveApi::updateParentFolder( $this->getBearerToken(), $args['id'], $args['new_parent'] );
            }
        } elseif( $job == 'upload_multipart' ) {
            if( ! empty( $args['path'] ) ) {
                if( $this->maybeRefreshToken() === false ) {
                    return false;
                }
                $res = GoogleDriveApi::uploadTypeMultipart( $this->getBearerToken(), $args['path'], $args['parent'] );
                if( is_array( $res ) && isset( $res['id'] ) ) {
                    GoogleDriveApi::addFilePermisson( $this->getBearerToken(), $res['id'] );
                    update_post_meta( $args['fb_id'], '_njfb_cloud_gg_drive_url', sprintf( 'https://drive.google.com/file/d/%s/view', $res['id'] ) );
                }
            }
        } elseif( $job == 'create' ) {
            if( $this->maybeRefreshToken() === false ) {
                return false;
            }
            $res = GoogleDriveApi::createFolder( $this->getBearerToken(), $args['name'], $args['parent'] );
            njfb_cloud_link_fb_to_cloud( 'google_drive', $args['fb_id'], $res['id'] );
        }
        return true;
    }
    private function getBearerToken()
    {
        $token = njfb_cloud_get_settings_by_key( 'google_drive', 'token', '' );
        return (is_array( $token ) && isset( $token['access_token'] )) ? $token['access_token'] : '';
    }
    private function maybeRefreshToken()
    {
        $token = njfb_cloud_get_settings_by_key( 'google_drive', 'token', '' );
        if( GoogleDriveApi::isExpired( $token ) && isset( $token['refresh_token'] ) ) {
            $new_token = GoogleDriveApi::refreshToken( $token['refresh_token'] );
            if( ! is_null( $new_token ) ) {
                $new_token = njfb_cloud_append_refresh_token( 'google_drive', $new_token );
                njfb_cloud_set_settings_by_key( 'google_drive', 'token', $new_token );
                return true;
            } else {
                return false;
            }
        }
        return true;
    }
}
