<?php
namespace NjFbCloud\Src\Model;

use NjFbCloud\Src\Interface\General;
use NjFbCloud\Src\Cloud\Cloud;

class FolderModel
{
    public static function getFolders( $cloud, $parent = null, $order_by = null, $limit = -1, $offset = 0 )
    {
        global $wpdb;
        $select = "ID AS id, post_title as name, post_content as cloud_id, post_parent as parent";

        $where = "post_type='".njfb_cloud_get_pt_name( $cloud )."'";
        if( ! is_null( $parent) ) {
            $where .= " AND post_parent = '".(int)$parent."'";
        }
        if( is_null( $order_by ) ) {
            $order_by = "post_title ASC, post_date_gmt DESC";
        }
        

        if( $limit  > -1 ) {
            $query = $wpdb->prepare( "SELECT {$select} FROM {$wpdb->posts} WHERE {$where} ORDER BY {$order_by} LIMIT %d, %d", $offset, $limit );
        } else {
            $query = $wpdb->prepare( "SELECT {$select} FROM {$wpdb->posts} WHERE {$where} ORDER BY {$order_by}");
        }
        return $wpdb->get_results( $query, ARRAY_A );
    }
    public static function insertFolder( $cloud, $folder  )
    {
        $folder = wp_parse_args( $folder, General::Folder );
        
        $parent = $folder['is_root'] ? 0 : self::findWpIdByCloudId( $cloud, $folder['parents'][0] );

        $data = array(
            'post_title' => $folder['name'],
            'post_content' => $folder['id'],
            'post_type' => njfb_cloud_get_pt_name( $cloud ),
            'post_parent' => $parent
        );

        if( isset( $folder['createdTime'] ) ) {
            $data['post_date'] = date( 'Y-m-d H:i:s', strtotime( $folder['createdTime'] ) );
            $data['post_date_gmt'] = date( 'Y-m-d H:i:s', strtotime( $folder['createdTime'] ) );
            // $data['post_date_gmt'] = get_gmt_from_date( date( 'Y-m-d H:i:s', strtotime( $folder['createdTime'] ) ) );
        }
        
        $id = wp_insert_post( $data );

        if( isset( $folder['modifiedTime'] ) ) {
            wp_update_post( array(
                'ID'                => $id, 
                'post_modified'     => date( 'Y-m-d H:i:s', strtotime( $folder['modifiedTime'] ) ),
                'post_modified_gmt' => date( 'Y-m-d H:i:s', strtotime( $folder['modifiedTime'] ) ),
                // 'post_modified_gmt' => get_gmt_from_date( date( 'Y-m-d H:i:s', strtotime( $folder['modifiedTime'] ) ) ),
            ) );
        }
        // ID as id, post_title as name, post_content as cloud_id, parent
        return array( 'id' => $id, 'name' => $folder['name'], 'cloud_id' => $folder['id'], 'parent' => $parent );
    }
    public static function findWpIdByCloudId( $cloud, $cloud_id )
    {
        global $wpdb;
        $query = $wpdb->prepare( "SELECT ID FROM {$wpdb->posts} WHERE post_content = %s AND post_type = %s", $cloud_id, njfb_cloud_get_pt_name( $cloud ) );
        return $wpdb->get_var( $query );
    }
    public static function getFolder( $cloud, $cloud_id )
    {
        global $wpdb;
        $res = $wpdb->get_row( $wpdb->prepare( "SELECT ID as id, post_title as name, post_content as cloud_id, parent FROM {$wpdb->posts} WHERE post_content = %s AND post_type = %s LIMIT 0,1 ", $cloud_id, njfb_cloud_get_pt_name( $cloud ) ), ARRAY_A );
        return $res;
    }
    public static function insertOrGet( $cloud, $folder)
    {
        $check = self::getFolder( $cloud, $folder['id']);
        if( is_null( $check ) ) {
            $check = self::insertFolder( $cloud, $folder);
            // $check = self::getFolder( $cloud, $folder['id']);
        }
        return $check;
    }
}
