<?php
function njfb_cloud_get_settings_by_group($group, $default = '') {
    global $njfb_cloud_settings;
    if( ! is_array( $njfb_cloud_settings ) || empty( $njfb_cloud_settings ) ) {
        $njfb_cloud_settings = get_option( 'njfb_cloud_settings', array() );
    }
    
    if( ! is_array( $njfb_cloud_settings ) ) {
        $njfb_cloud_settings = array();
    }
    if( isset( $njfb_cloud_settings[$group] ) ) {
        return $njfb_cloud_settings[$group];
    }
    return $default;
}
function njfb_cloud_get_settings_by_key($group, $key, $default = '') {
    global $njfb_cloud_settings;
    if( ! is_array( $njfb_cloud_settings ) || empty( $njfb_cloud_settings ) ) {
        $njfb_cloud_settings = get_option( 'njfb_cloud_settings', array() );
    }
    
    if( ! is_array( $njfb_cloud_settings ) ) {
        $njfb_cloud_settings = array();
    }
    if( isset( $njfb_cloud_settings[$group] ) ) {
        if( isset( $njfb_cloud_settings[$group][$key] ) ) {
            if( $group == 'google_drive' && $key == 'token' ) {
                $njfb_cloud_settings[$group][$key] = wp_parse_args( $njfb_cloud_settings[$group][$key], NjFbCloud\Src\Interface\Google::Token );
            }
            return $njfb_cloud_settings[$group][$key];
        }
        return $default;
    }
    return $default;
}
function njfb_cloud_set_settings_by_key($group, $key, $value) {
    global $njfb_cloud_settings;
    if( ! is_array( $njfb_cloud_settings ) || empty( $njfb_cloud_settings ) ) {
        $njfb_cloud_settings = get_option( 'njfb_cloud_settings', array() );
    }
    
    if( ! is_array( $njfb_cloud_settings ) ) {
        $njfb_cloud_settings = array();
    }
    if( $group == 'google_drive' && $key == 'token' ) {
        $value = wp_parse_args( $value, NjFbCloud\Src\Interface\Google::Token );
    }
    $njfb_cloud_settings[$group][$key] = $value;
    
    update_option( 'njfb_cloud_settings', $njfb_cloud_settings );
}
function njfb_cloud_ggdrive_jsorigins()
{
    return home_url( '/' );
}
function njfb_cloud_ggdrive_redirecturis()
{
    return add_query_arg( array('action' => 'njfb_cloud_ggdrive_connected' ), admin_url( 'admin-ajax.php' ) );
}

function njfb_cloud_selected_tab()
{
    $selected_tab = isset( $_GET['tab'] ) ? sanitize_key( $_GET['tab'] ) : '';
    if( empty( $selected_tab ) ) {
        $selected_tab = 'google_drive';
    }
    return $selected_tab;
}
function njfb_cloud_append_refresh_token( $cloud, $token ) {
    if( ! isset( $token['refresh_token'] ) ) {
        $old_token = njfb_cloud_get_settings_by_key($cloud, 'token', array() );
        if( ! is_array( $old_token ) ) {
            $old_token = array();
        }
        $token['refresh_token'] = isset( $old_token['refresh_token'] ) ? $old_token['refresh_token'] : '';
    }
    return $token;
}

function njfb_cloud_is_cloud_valid( $cloud )
{
    global $njfb_cloud_config;
    return in_array( $cloud, array_keys( $njfb_cloud_config['supported_cloud'] ) );
}

/**
 * Links a FileBird folder to a cloud storage folder.
 *
 * This function updates the WordPress options table to create a link between 
 * a FileBird folder and a corresponding cloud storage folder. It stores the 
 * association using a unique option name constructed from the provided 
 * parameters.
 *
 * @param string $cloud The name of the cloud storage provider.
 * @param int    $filebird_id The ID of the FileBird folder.
 * @param string $cloud_id The ID of the folder in the cloud storage.
 */
function njfb_cloud_link_fb_to_cloud( $cloud, $filebird_id, $cloud_id )
{
    update_option( 'njfb_cloud_folder_' . $cloud . '_' . $filebird_id, $cloud_id );
}
function njfb_cloud_remove_link_fb( $cloud, $filebird_id )
{
    delete_option( 'njfb_cloud_folder_' . $cloud . '_' . $filebird_id );
}
function njfb_cloud_get_cloud_id_from_fb( $cloud, $filebird_id )
{
    return get_option( 'njfb_cloud_folder_' . $cloud . '_' . $filebird_id, '' );
}

/**
 * Retrieves the main folder ID for a specific cloud storage provider.
 *
 * This function fetches the main folder ID associated with a given cloud 
 * storage provider from the WordPress options table. If the option is not 
 * found, it returns an empty string.
 *
 * @param string $cloud The name of the cloud storage provider.
 * @return string The main folder ID for the specified cloud storage provider, or an empty string if not set.
 */
function njfb_cloud_get_main_folder( $cloud ) {
    return get_option( 'njfb_cloud_' . $cloud . '_main_folder', '' );
}

/**
 * Sets the main folder ID for a specific cloud storage provider.
 *
 * This function updates the WordPress options table to set the main folder ID 
 * for a given cloud storage provider.
 *
 * @param string $cloud The name of the cloud storage provider.
 * @param string $cloud_id The ID of the main folder in the cloud storage.
 */

function njfb_cloud_set_main_folder( $cloud, $cloud_id ) {
    update_option( 'njfb_cloud_' . $cloud . '_main_folder', $cloud_id );
}

/**
 * Deletes the main folder ID for a specific cloud storage provider.
 *
 * This function removes the option that stores the main folder ID for a given 
 * cloud storage provider from the WordPress options table.
 *
 * @param string $cloud The name of the cloud storage provider.
 */
function njfb_cloud_delete_main_folder( $cloud ) {
    delete_option( 'njfb_cloud_' . $cloud . '_main_folder' );
}

function njfb_cloud_get_fb_id_from_cloud( $cloud, $cloud_id )
{
    global $wpdb;
    $key = $wpdb->get_var( $wpdb->prepare( "SELECT `option_key` FROM {$wpdb->options} WHERE `option_value` = %s", $cloud_id ) );
    $ex = explode( 'njfb_cloud_folder_' . $cloud . '_', $key );
    if( count( $ex ) == 2 ) {
        return (int)$ex[1];
    }
    return null;
}
function njfb_cloud_clear_options( $cloud ) {
    global $wpdb;
    $wpdb->query( "DELETE FROM $wpdb->options WHERE option_name LIKE 'njfb_cloud_folder_" . $cloud . "_%'" );
}
/**
 * Retrieves the domain name of the WordPress site.
 *
 * This function fetches the site URL and removes the protocol (http or https) 
 * to return the domain name. If the site URL is empty or null, it falls back 
 * to using the home URL.
 *
 * @return string The domain name of the WordPress site.
 */
function njfb_cloud_get_domain()
{
    $url = get_site_url();
    if ( $url == '' || $url == null ) {
        $url = home_url();
    }
    return preg_replace( '#https?:\/\/#', '', $url );
}
function njfb_cloud_dispatch($class, $args = array())
{
    global $wpdb;
    update_option( 'njfb_cloud_job_'. $class .'_' . uniqid(), compact( array( 'class', 'args' ) ), false );
}
function njfb_cloud_clear_jobs( $class ) {
    global $wpdb;
    $wpdb->query( "DELETE FROM $wpdb->options WHERE option_name LIKE 'njfb_cloud_job_" . $class . "_%'" );
}
function njfb_cloud_queue_work()
{
    global $wpdb;
    $oldest_job = $wpdb->get_row( $wpdb->prepare( "SELECT `option_id`, `option_value` FROM {$wpdb->options} WHERE `option_name` LIKE 'njfb_cloud_job_%' ORDER BY `option_id` ASC" ), ARRAY_A );
    if( is_array( $oldest_job ) ) {
        $option_value = maybe_unserialize( $oldest_job['option_value'] );
        if( is_array( $option_value ) ) {
            $class = 'NjFbCloud\\Src\\Jobs\\' . $option_value['class'];
            $args = $option_value['args'];
            $job = new $class( $args );
            $res = $job->handle();
            if( $res !== false ) {
                $wpdb->delete( $wpdb->options, array( 'option_id' => $oldest_job['option_id'] ), array( '%d' ) );
            }
        }
    }
    return false;
}
function njfb_cloud_get_ggdrive_id( $gg_drive_url ) {
    preg_match( '#https://drive.google.com/file/d/(.*?)/view#', $gg_drive_url, $m );
    return isset( $m[1] ) ? $m[1] : null;
}

/**
 * Logs a message to a daily log file.
 *
 * This function appends a message to a log file named with the current date 
 * and an MD5 hash of the WordPress AUTH_KEY. Each log entry is prefixed with 
 * a timestamp.
 *
 * @param string $mess The message to log.
 */
function njfb_cloud_log( $mess )
{
    $key = md5( AUTH_KEY );
    $filename = date('d-m-Y') . '-' . $key . '.txt';
    $mess = date('d-m-Y H:i:s'). "\n" . $mess . "\n";
    file_put_contents( NJFB_CLOUD_PATH . 'logs/' . $filename, $mess, FILE_APPEND );
}

function njfb_cloud_dropbox_redirecturis()
{
    return add_query_arg( array('action' => 'njfb_cloud_dropbox_connected' ), admin_url( 'admin-ajax.php' ) );
}
function njfb_cloud_save_ggdrive_sub( $sub ) {
    update_option( 'njfb_cloud_ggdrive_sub', $sub );
}

function njfb_cloud_get_last_message() {
    $saved = get_option( 'njt_fb_cloud_last_message', array() );
    if( ! is_array( $saved ) ) {
        $saved = array();
    }
    return $saved;
}
function njfb_cloud_set_last_message( $mess ) {
    $saved = njfb_cloud_get_last_message();
    $saved[] = $mess;
    update_option( 'njt_fb_cloud_last_message', $saved );
}
function njfb_cloud_remove_last_message( $mess ) {
    delete_option( 'njt_fb_cloud_last_message' );
}
function njfb_cloud_close_connection( $arr ) {
    ob_start();

    echo wp_json_encode( $arr );

    header('Connection: close');
    header('Content-Length: '.ob_get_length());
    header('Content-Encoding: none');
    ob_end_flush();
    @ob_flush();
    flush();
    if( function_exists( 'fastcgi_finish_request' ) ) {
        \fastcgi_finish_request();
    }
    if( function_exists( 'litespeed_finish_request' ) ) {
        \litespeed_finish_request();
    }
}
